@@ -120,7 +120,7 @@ GST_START_TEST (test_audio_per_byte)
   pk = gst_dp_packetizer_new (GST_DP_VERSION_1_0);
 
   gdpdepay = setup_gdpdepay ();
-  srcpad = gst_element_get_pad (gdpdepay, "src");
+  srcpad = gst_element_get_static_pad (gdpdepay, "src");
 
   fail_unless (gst_element_set_state (gdpdepay,
           GST_STATE_PLAYING) == GST_STATE_CHANGE_SUCCESS,
@@ -196,7 +196,7 @@ GST_START_TEST (test_audio_in_one_buffer)
   pk = gst_dp_packetizer_new (GST_DP_VERSION_1_0);
 
   gdpdepay = setup_gdpdepay ();
-  srcpad = gst_element_get_pad (gdpdepay, "src");
+  srcpad = gst_element_get_static_pad (gdpdepay, "src");
 
   fail_unless (gst_element_set_state (gdpdepay,
           GST_STATE_PLAYING) == GST_STATE_CHANGE_SUCCESS,
@@ -279,8 +279,6 @@ setup_gdpdepay_streamheader (void)
   return gdpdepay;
 }
 
-#ifndef HAVE_CPU_PPC64          /* Test known to fail on PPC64. See #348114 */
-
 /* this tests deserialization of a GDP stream where the serialized caps
  * have a streamheader set */
 GST_START_TEST (test_streamheader)
@@ -300,7 +298,7 @@ GST_START_TEST (test_streamheader)
   pk = gst_dp_packetizer_new (GST_DP_VERSION_1_0);
 
   gdpdepay = setup_gdpdepay_streamheader ();
-  srcpad = gst_element_get_pad (gdpdepay, "src");
+  srcpad = gst_element_get_static_pad (gdpdepay, "src");
   ASSERT_OBJECT_REFCOUNT (gdpdepay, "gdpdepay", 1);
 
   fail_unless (gst_element_set_state (gdpdepay,
@@ -398,8 +396,6 @@ GST_START_TEST (test_streamheader)
 
 GST_END_TEST;
 
-#endif /* ifndef HAVE_CPU_PPC64 */
-
 static Suite *
 gdpdepay_suite (void)
 {
@@ -409,9 +405,8 @@ gdpdepay_suite (void)
   suite_add_tcase (s, tc_chain);
   tcase_add_test (tc_chain, test_audio_per_byte);
   tcase_add_test (tc_chain, test_audio_in_one_buffer);
-#ifndef HAVE_CPU_PPC64          /* Test known to fail on PPC64. See #348114 */
   tcase_add_test (tc_chain, test_streamheader);
-#endif
+
   return s;
 }
 
