@@ -25,13 +25,12 @@
 #include <gst/base/gstbasetransform.h>
 #include <gst/check/gstcheck.h>
 #include <gst/controller/gstcontroller.h>
-
-gboolean have_eos = FALSE;
+#include <gst/interfaces/streamvolume.h>
 
 /* For ease of programming we use globals to keep refs for our floating
  * src and sink pads we create; otherwise we always have to do get_pad,
  * get_peer, and then remove references in every test function */
-GstPad *mysrcpad, *mysinkpad;
+static GstPad *mysrcpad, *mysinkpad;
 
 
 #define VOLUME_CAPS_TEMPLATE_STRING     \
@@ -119,8 +118,8 @@ static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
     GST_STATIC_CAPS (VOLUME_CAPS_TEMPLATE_STRING)
     );
 
-GstElement *
-setup_volume ()
+static GstElement *
+setup_volume (void)
 {
   GstElement *volume;
 
@@ -134,7 +133,7 @@ setup_volume ()
   return volume;
 }
 
-void
+static void
 cleanup_volume (GstElement * volume)
 {
   GST_DEBUG ("cleanup_volume");
@@ -150,6 +149,35 @@ cleanup_volume (GstElement * volume)
   gst_check_teardown_element (volume);
 }
 
+GST_START_TEST (test_get_set)
+{
+  GstElement *volume = gst_element_factory_make ("volume", NULL);
+  gdouble val;
+
+  fail_unless (volume != NULL);
+  g_object_get (G_OBJECT (volume), "volume", &val, NULL);
+  fail_unless (val == 1.0);
+  fail_unless (val == gst_stream_volume_get_volume (GST_STREAM_VOLUME (volume),
+          GST_STREAM_VOLUME_FORMAT_LINEAR));
+
+  g_object_set (G_OBJECT (volume), "volume", 0.5, NULL);
+  g_object_get (G_OBJECT (volume), "volume", &val, NULL);
+  fail_unless (val == 0.5);
+  fail_unless (val == gst_stream_volume_get_volume (GST_STREAM_VOLUME (volume),
+          GST_STREAM_VOLUME_FORMAT_LINEAR));
+
+  gst_stream_volume_set_volume (GST_STREAM_VOLUME (volume),
+      GST_STREAM_VOLUME_FORMAT_LINEAR, 1.0);
+  g_object_get (G_OBJECT (volume), "volume", &val, NULL);
+  fail_unless (val == 1.0);
+  fail_unless (val == gst_stream_volume_get_volume (GST_STREAM_VOLUME (volume),
+          GST_STREAM_VOLUME_FORMAT_LINEAR));
+
+  gst_object_unref (volume);
+}
+
+GST_END_TEST;
+
 GST_START_TEST (test_unity_s8)
 {
   GstElement *volume;
@@ -1336,8 +1364,7 @@ GST_START_TEST (test_passthrough)
   gint16 *res;
 
   volume = setup_volume ();
-  g_object_set (G_OBJECT (volume), "volume", 2.0, NULL);
-  gst_base_transform_set_passthrough (GST_BASE_TRANSFORM (volume), TRUE);
+  g_object_set (G_OBJECT (volume), "volume", 1.0, NULL);
   fail_unless (gst_element_set_state (volume,
           GST_STATE_PLAYING) == GST_STATE_CHANGE_SUCCESS,
       "could not set to playing");
@@ -1403,6 +1430,58 @@ GST_START_TEST (test_controller_usability)
 
 GST_END_TEST;
 
+GST_START_TEST (test_controller_processing)
+{
+  GstInterpolationControlSource *csource;
+  GstController *c;
+  GstElement *volume;
+  GstBuffer *inbuffer, *outbuffer;
+  GstCaps *caps;
+  gint16 in[2] = { 16384, -256 };
+  gint16 *res;
+
+  volume = setup_volume ();
+
+  c = gst_controller_new (G_OBJECT (volume), "volume", NULL);
+
+  fail_unless (GST_IS_CONTROLLER (c));
+
+  csource = gst_interpolation_control_source_new ();
+  gst_interpolation_control_source_set_interpolation_mode (csource,
+      GST_INTERPOLATE_CUBIC);
+  gst_controller_set_control_source (c, "volume", GST_CONTROL_SOURCE (csource));
+  g_object_unref (csource);
+
+  fail_unless (gst_element_set_state (volume,
+          GST_STATE_PLAYING) == GST_STATE_CHANGE_SUCCESS,
+      "could not set to playing");
+
+  inbuffer = gst_buffer_new_and_alloc (4);
+  memcpy (GST_BUFFER_DATA (inbuffer), in, 4);
+  caps = gst_caps_from_string (VOLUME_CAPS_STRING_S16);
+  gst_buffer_set_caps (inbuffer, caps);
+  GST_BUFFER_TIMESTAMP (inbuffer) = 0;
+  gst_caps_unref (caps);
+  ASSERT_BUFFER_REFCOUNT (inbuffer, "inbuffer", 1);
+
+  /* pushing gives away my reference ... */
+  fail_unless (gst_pad_push (mysrcpad, inbuffer) == GST_FLOW_OK);
+  /* ... but it ends up being collected on the global buffer list */
+  ASSERT_BUFFER_REFCOUNT (inbuffer, "inbuffer", 1);
+  fail_unless_equals_int (g_list_length (buffers), 1);
+  fail_if ((outbuffer = (GstBuffer *) buffers->data) == NULL);
+  fail_unless (inbuffer == outbuffer);
+  res = (gint16 *) GST_BUFFER_DATA (outbuffer);
+  GST_INFO ("expected %+5d %+5d  real %+5d %+5d", in[0], in[1], res[0], res[1]);
+  fail_unless (memcmp (GST_BUFFER_DATA (inbuffer), in, 4) == 0);
+
+  g_object_unref (c);
+
+  cleanup_volume (volume);
+}
+
+GST_END_TEST;
+
 static Suite *
 volume_suite (void)
 {
@@ -1410,6 +1489,7 @@ volume_suite (void)
   TCase *tc_chain = tcase_create ("general");
 
   suite_add_tcase (s, tc_chain);
+  tcase_add_test (tc_chain, test_get_set);
   tcase_add_test (tc_chain, test_unity_s8);
   tcase_add_test (tc_chain, test_half_s8);
   tcase_add_test (tc_chain, test_double_s8);
@@ -1437,6 +1517,7 @@ volume_suite (void)
   tcase_add_test (tc_chain, test_wrong_caps);
   tcase_add_test (tc_chain, test_passthrough);
   tcase_add_test (tc_chain, test_controller_usability);
+  tcase_add_test (tc_chain, test_controller_processing);
 
   return s;
 }
