@@ -270,25 +270,98 @@ gst_cd_foo_src_read_sector (GstCddaBaseSrc * cddabasesrc, gint sector)
   return buf;
 }
 
-GST_START_TEST (test_discid_calculations)
+static inline gboolean
+tag_list_has_tag (GstTagList * list, const gchar * tag, GType type)
+{
+  const GValue *val = gst_tag_list_get_value_index (list, tag, 0);
+
+  if (val == NULL) {
+    GST_LOG ("no tag '%s' in taglist %" GST_PTR_FORMAT, tag, list);
+    return FALSE;
+  }
+
+  if (!G_VALUE_HOLDS (val, type)) {
+    GST_LOG ("tag '%s' in taglist %" GST_PTR_FORMAT " is not of type %s",
+        tag, list, g_type_name (type));
+    return FALSE;
+  }
+
+  return TRUE;
+}
+
+static void
+test_uri_parse (const gchar * uri, const gchar * device, gint track)
 {
   GstElement *foosrc;
+  gchar *set_device = NULL;
+  gint set_track = 0;
+
+  foosrc = gst_element_factory_make ("cdfoosrc", "cdfoosrc");
+  fail_unless (gst_uri_handler_set_uri (GST_URI_HANDLER (foosrc), uri) == TRUE,
+      "couldn't set uri %s", uri);
+  g_object_get (foosrc, "device", &set_device, "track", &set_track, NULL);
+  fail_unless (set_device != NULL);
+  fail_unless (strcmp (set_device, device) == 0,
+      "device set was %s, expected %s", set_device, device);
+  fail_unless (set_track == track, "track set was %d, expected %d", set_track,
+      track);
+  g_free (set_device);
+  gst_object_unref (foosrc);
+}
+
+GST_START_TEST (test_discid_calculations)
+{
+  GstElement *foosrc, *pipeline, *sink;
   gint i;
 
   fail_unless (gst_element_register (NULL, "cdfoosrc", GST_RANK_SECONDARY,
           GST_TYPE_CD_FOO_SRC));
 
+  pipeline = gst_pipeline_new ("pipeline");
+
+  sink = gst_element_factory_make ("fakesink", "sink");
+  fail_unless (sink != NULL, "couldn't create fakesink");
+
   foosrc = gst_element_factory_make ("cdfoosrc", "cdfoosrc");
+  fail_unless (foosrc != NULL, "couldn't create cdfoosrc");
+
+  gst_bin_add (GST_BIN (pipeline), foosrc);
+  gst_bin_add (GST_BIN (pipeline), sink);
+  fail_unless (gst_element_link (foosrc, sink));
 
   for (i = 0; i < G_N_ELEMENTS (test_discs); ++i) {
+    GstTagList *tags = NULL;
+    GstMessage *msg;
+
     GST_LOG ("Testing disc layout %u ...", i);
     GST_CD_FOO_SRC (foosrc)->cur_disc = i;
-    gst_element_set_state (foosrc, GST_STATE_PLAYING);
-    gst_element_get_state (foosrc, NULL, NULL, -1);
-    gst_element_set_state (foosrc, GST_STATE_NULL);
+    gst_element_set_state (pipeline, GST_STATE_PLAYING);
+
+    msg =
+        gst_bus_timed_pop_filtered (GST_ELEMENT_BUS (pipeline),
+        GST_CLOCK_TIME_NONE, GST_MESSAGE_TAG);
+    gst_message_parse_tag (msg, &tags);
+    fail_unless (tags != NULL);
+    fail_unless (tag_list_has_tag (tags, "track-count", G_TYPE_UINT));
+    fail_unless (tag_list_has_tag (tags, "track-number", G_TYPE_UINT));
+    fail_unless (tag_list_has_tag (tags, "duration", G_TYPE_UINT64));
+    fail_unless (tag_list_has_tag (tags, "discid", G_TYPE_STRING));
+    fail_unless (tag_list_has_tag (tags, "discid-full", G_TYPE_STRING));
+    fail_unless (tag_list_has_tag (tags, "musicbrainz-discid", G_TYPE_STRING));
+    fail_unless (tag_list_has_tag (tags, "musicbrainz-discid-full",
+            G_TYPE_STRING));
+    gst_tag_list_free (tags);
+    gst_message_unref (msg);
+
+    msg =
+        gst_bus_timed_pop_filtered (GST_ELEMENT_BUS (pipeline),
+        GST_CLOCK_TIME_NONE, GST_MESSAGE_ASYNC_DONE);
+    gst_message_unref (msg);
+
+    gst_element_set_state (pipeline, GST_STATE_NULL);
   }
 
-  gst_object_unref (foosrc);
+  gst_object_unref (pipeline);
 
   gst_task_cleanup_all ();
 }
@@ -348,6 +421,45 @@ GST_START_TEST (test_buffer_timestamps)
 
 GST_END_TEST;
 
+GST_START_TEST (test_uri_parsing)
+{
+  GstElement *foosrc;
+
+  fail_unless (gst_element_register (NULL, "cdfoosrc", GST_RANK_SECONDARY,
+          GST_TYPE_CD_FOO_SRC));
+
+  /* wrong protocol */
+  foosrc = gst_element_factory_make ("cdfoosrc", "cdfoosrc");
+  fail_unless (gst_uri_handler_set_uri (GST_URI_HANDLER (foosrc),
+          "x://") == FALSE);
+  fail_unless (gst_uri_handler_set_uri (GST_URI_HANDLER (foosrc),
+          "cddaq://") == FALSE);
+
+  /* cdda://track */
+  test_uri_parse ("cdda://", "/dev/cdrom", 1);
+  test_uri_parse ("cdda://2", "/dev/cdrom", 2);
+  test_uri_parse ("cdda://47", "/dev/cdrom", 47);
+  fail_unless (gst_uri_handler_set_uri (GST_URI_HANDLER (foosrc),
+          "cdda://-1") == FALSE);
+  fail_unless (gst_uri_handler_set_uri (GST_URI_HANDLER (foosrc),
+          "cdda://what") == FALSE);
+
+  /* cdda://device#track */
+  test_uri_parse ("cdda:///dev/hdb#1", "/dev/hdb", 1);
+  test_uri_parse ("cdda://anything#8", "anything", 8);
+  fail_unless (gst_uri_handler_set_uri (GST_URI_HANDLER (foosrc),
+          "cdda:///dev/hdb#nonsense") == FALSE);
+  fail_unless (gst_uri_handler_set_uri (GST_URI_HANDLER (foosrc),
+          "cdda:///dev/hdb#-2") == FALSE);
+
+  /* cdda://track#device (device should be ignored - FIXME 0.11) */
+  test_uri_parse ("cdda://8#/dev/hdb", "/dev/cdrom", 8);
+
+  gst_object_unref (foosrc);
+}
+
+GST_END_TEST;
+
 static Suite *
 cddabasesrc_suite (void)
 {
@@ -357,6 +469,7 @@ cddabasesrc_suite (void)
   suite_add_tcase (s, tc_chain);
   tcase_add_test (tc_chain, test_discid_calculations);
   tcase_add_test (tc_chain, test_buffer_timestamps);
+  tcase_add_test (tc_chain, test_uri_parsing);
 
   return s;
 }
