@@ -191,6 +191,7 @@ GST_START_TEST (test_muscibrainz_tag_registration)
   gst_vorbis_tag_add (list, "MUSICBRAINZ_ALBUMID", "345678");
   gst_vorbis_tag_add (list, "MUSICBRAINZ_ALBUMARTISTID", "4567890");
   gst_vorbis_tag_add (list, "MUSICBRAINZ_TRMID", "5678901");
+  /* MUSICBRAINZ_SORTNAME = GST_TAG_ARTIST_SORTNAME now */
   gst_vorbis_tag_add (list, "MUSICBRAINZ_SORTNAME", "Five, 678901");
 
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_MUSICBRAINZ_TRACKID, "123456");
@@ -199,8 +200,7 @@ GST_START_TEST (test_muscibrainz_tag_registration)
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_MUSICBRAINZ_ALBUMARTISTID,
       "4567890");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_MUSICBRAINZ_TRMID, "5678901");
-  ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_MUSICBRAINZ_SORTNAME,
-      "Five, 678901");
+  ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_ARTIST_SORTNAME, "Five, 678901");
 
   gst_tag_list_free (list);
 }
@@ -258,6 +258,8 @@ GST_START_TEST (test_vorbis_tags)
   gst_vorbis_tag_add (list, "COPYRIGHT", "Copyfoo");
   gst_vorbis_tag_add (list, "DESCRIPTION", "Descoo");
   gst_vorbis_tag_add (list, "LICENSE", "Licoo");
+  gst_vorbis_tag_add (list, "LICENSE",
+      "http://creativecommons.org/licenses/by/3.0/");
   gst_vorbis_tag_add (list, "LOCATION", "Bristol, UK");
   gst_vorbis_tag_add (list, "ORGANIZATION", "Orgoo");
   gst_vorbis_tag_add (list, "GENRE", "Goo");
@@ -278,7 +280,9 @@ GST_START_TEST (test_vorbis_tags)
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_COPYRIGHT, "Copyfoo");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_DESCRIPTION, "Descoo");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_LICENSE, "Licoo");
-  ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_LOCATION, "Bristol, UK");
+  ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_LICENSE_URI,
+      "http://creativecommons.org/licenses/by/3.0/");
+  ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_GEO_LOCATION_NAME, "Bristol, UK");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_ORGANIZATION, "Orgoo");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_GENRE, "Goo");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_CONTACT, "Coo");
@@ -322,6 +326,7 @@ GST_START_TEST (test_vorbis_tags)
    * (',' or '.') regardless of the current locale */
   gst_vorbis_tag_add (list, "REPLAYGAIN_ALBUM_PEAK", "0,98107");
   ASSERT_TAG_LIST_HAS_DOUBLE (list, GST_TAG_ALBUM_PEAK, 0.98107);
+  gst_vorbis_tag_add (list, "LICENSE", "http://foo.com/license-1.html");
 
   /* make sure we can convert back and forth without loss */
   {
@@ -582,6 +587,8 @@ GST_START_TEST (test_id3_tags)
   fail_unless (gst_tag_from_id3_tag ("TALB") != NULL);
   ASSERT_CRITICAL (gst_tag_from_id3_tag (NULL));
   fail_unless (gst_tag_from_id3_tag ("R2D2") == NULL);
+  fail_unless_equals_string (gst_tag_from_id3_tag ("WCOP"),
+      GST_TAG_COPYRIGHT_URI);
 
   /* gst_tag_from_id3_user_tag */
   ASSERT_CRITICAL (gst_tag_from_id3_user_tag (NULL, "foo"));
@@ -592,7 +599,8 @@ GST_START_TEST (test_id3_tags)
   ASSERT_CRITICAL (gst_tag_to_id3_tag (NULL));
   fail_unless (gst_tag_to_id3_tag ("R2D2") == NULL);
   fail_unless (gst_tag_to_id3_tag (GST_TAG_ARTIST) != NULL);
-
+  fail_unless_equals_string (gst_tag_to_id3_tag (GST_TAG_COPYRIGHT_URI),
+      "WCOP");
 
   fail_unless (GST_TYPE_TAG_IMAGE_TYPE != 0);
   fail_unless (g_type_name (GST_TYPE_TAG_IMAGE_TYPE) != NULL);
