@@ -21,90 +21,12 @@
  */
 
 #include <gst/check/gstcheck.h>
+#include <gst/check/gstbufferstraw.h>
+
+#ifndef GST_DISABLE_PARSE
 
 #define TIMESTAMP_OFFSET G_GINT64_CONSTANT(3249870963)
 
-static GCond *cond = NULL;
-static GMutex *lock = NULL;
-static GstBuffer *buf = NULL;
-static gulong id;
-
-static gboolean
-buffer_probe (GstPad * pad, GstBuffer * buffer, gpointer unused)
-{
-  g_mutex_lock (lock);
-
-  while (buf != NULL)
-    g_cond_wait (cond, lock);
-
-  buf = gst_buffer_ref (buffer);
-
-  g_cond_signal (cond);
-
-  g_mutex_unlock (lock);
-
-  return TRUE;
-}
-
-static void
-start_pipeline (GstElement * bin, GstPad * pad)
-{
-  id = gst_pad_add_buffer_probe (pad, G_CALLBACK (buffer_probe), NULL);
-
-  cond = g_cond_new ();
-  lock = g_mutex_new ();
-
-  gst_element_set_state (bin, GST_STATE_PLAYING);
-
-}
-
-static GstBuffer *
-get_buffer (GstElement * bin, GstPad * pad)
-{
-  GstBuffer *ret;
-
-  g_mutex_lock (lock);
-
-  while (buf == NULL)
-    g_cond_wait (cond, lock);
-
-  ret = buf;
-  buf = NULL;
-
-  g_cond_signal (cond);
-
-  g_mutex_unlock (lock);
-
-  return ret;
-}
-
-static void
-stop_pipeline (GstElement * bin, GstPad * pad)
-{
-  g_mutex_lock (lock);
-  if (buf)
-    gst_buffer_unref (buf);
-  buf = NULL;
-  gst_pad_remove_buffer_probe (pad, (guint) id);
-  id = 0;
-  g_cond_signal (cond);
-  g_mutex_unlock (lock);
-
-  gst_element_set_state (bin, GST_STATE_NULL);
-
-  g_mutex_lock (lock);
-  if (buf)
-    gst_buffer_unref (buf);
-  buf = NULL;
-  g_mutex_unlock (lock);
-
-  g_mutex_free (lock);
-  g_cond_free (cond);
-
-  lock = NULL;
-  cond = NULL;
-}
-
 static void
 check_buffer_timestamp (GstBuffer * buffer, GstClockTime timestamp)
 {
@@ -124,12 +46,24 @@ check_buffer_duration (GstBuffer * buffer, GstClockTime duration)
 }
 
 static void
-check_buffer_granulepos (GstBuffer * buffer, GstClockTime granulepos)
+check_buffer_granulepos (GstBuffer * buffer, gint64 granulepos)
 {
+  GstClockTime clocktime;
+
   fail_unless (GST_BUFFER_OFFSET_END (buffer) == granulepos,
       "expected granulepos %" G_GUINT64_FORMAT
       ", but got granulepos %" G_GUINT64_FORMAT,
       granulepos, GST_BUFFER_OFFSET_END (buffer));
+
+  /* contrary to what we record as TIMESTAMP, we can use OFFSET to check
+   * the granulepos correctly here */
+  clocktime = gst_util_uint64_scale (GST_BUFFER_OFFSET_END (buffer), 44100,
+      GST_SECOND);
+
+  fail_unless (clocktime == GST_BUFFER_OFFSET (buffer),
+      "expected OFFSET set to clocktime %" GST_TIME_FORMAT
+      ", but got %" GST_TIME_FORMAT,
+      GST_TIME_ARGS (clocktime), GST_TIME_ARGS (GST_BUFFER_OFFSET (buffer)));
 }
 
 /* this check is here to check that the granulepos we derive from the timestamp
@@ -140,7 +74,7 @@ check_buffer_granulepos (GstBuffer * buffer, GstClockTime granulepos)
 static void
 check_buffer_granulepos_from_endtime (GstBuffer * buffer, GstClockTime endtime)
 {
-  GstClockTime granulepos, expected;
+  gint64 granulepos, expected;
 
   granulepos = GST_BUFFER_OFFSET_END (buffer);
   expected = gst_util_uint64_scale (endtime, 44100, GST_SECOND);
@@ -159,7 +93,6 @@ GST_START_TEST (test_granulepos_offset)
   gchar *pipe_str;
   GstBuffer *buffer;
   GError *error = NULL;
-  GstClockTime timestamp;
 
   pipe_str = g_strdup_printf ("audiotestsrc timestamp-offset=%" G_GUINT64_FORMAT
       " ! audio/x-raw-int,rate=44100"
@@ -175,34 +108,37 @@ GST_START_TEST (test_granulepos_offset)
     GstElement *sink = gst_bin_get_by_name (GST_BIN (bin), "fakesink0");
 
     fail_unless (sink != NULL, "Could not get fakesink out of bin");
-    pad = gst_element_get_pad (sink, "sink");
+    pad = gst_element_get_static_pad (sink, "sink");
     fail_unless (pad != NULL, "Could not get pad out of fakesink");
     gst_object_unref (sink);
   }
 
-  start_pipeline (bin, pad);
+  gst_buffer_straw_start_pipeline (bin, pad);
 
   /* header packets should have timestamp == NONE, granulepos 0 */
-  buffer = get_buffer (bin, pad);
+  buffer = gst_buffer_straw_get_buffer (bin, pad);
+  GST_DEBUG ("Got buffer in test");
+  check_buffer_timestamp (buffer, GST_CLOCK_TIME_NONE);
+  check_buffer_duration (buffer, GST_CLOCK_TIME_NONE);
+  check_buffer_granulepos (buffer, 0);
+  gst_buffer_unref (buffer);
+  GST_DEBUG ("Unreffed buffer in test");
+
+  buffer = gst_buffer_straw_get_buffer (bin, pad);
   check_buffer_timestamp (buffer, GST_CLOCK_TIME_NONE);
   check_buffer_duration (buffer, GST_CLOCK_TIME_NONE);
   check_buffer_granulepos (buffer, 0);
   gst_buffer_unref (buffer);
 
-  buffer = get_buffer (bin, pad);
-  check_buffer_timestamp (buffer, GST_CLOCK_TIME_NONE);
-  check_buffer_duration (buffer, GST_CLOCK_TIME_NONE);
-  check_buffer_granulepos (buffer, 0);
-  gst_buffer_unref (buffer);
-
-  buffer = get_buffer (bin, pad);
+  buffer = gst_buffer_straw_get_buffer (bin, pad);
   check_buffer_timestamp (buffer, GST_CLOCK_TIME_NONE);
   check_buffer_duration (buffer, GST_CLOCK_TIME_NONE);
   check_buffer_granulepos (buffer, 0);
   gst_buffer_unref (buffer);
 
   {
-    GstClockTime next_timestamp, last_granulepos;
+    GstClockTime next_timestamp;
+    gint64 last_granulepos = 0;
 
     /* first buffer should have timestamp of TIMESTAMP_OFFSET, granulepos to
      * match the timestamp of the end of the last sample in the output buffer.
@@ -210,7 +146,7 @@ GST_START_TEST (test_granulepos_offset)
      * value due to loss of precision with granulepos. vorbisenc does take care
      * to timestamp correctly based on the offset of the input data however, so
      * it does do sub-granulepos timestamping. */
-    buffer = get_buffer (bin, pad);
+    buffer = gst_buffer_straw_get_buffer (bin, pad);
     last_granulepos = GST_BUFFER_OFFSET_END (buffer);
     check_buffer_timestamp (buffer, TIMESTAMP_OFFSET);
     /* don't really have a good way of checking duration... */
@@ -222,7 +158,7 @@ GST_START_TEST (test_granulepos_offset)
     gst_buffer_unref (buffer);
 
     /* check continuity with the next buffer */
-    buffer = get_buffer (bin, pad);
+    buffer = gst_buffer_straw_get_buffer (bin, pad);
     check_buffer_timestamp (buffer, next_timestamp);
     check_buffer_duration (buffer,
         gst_util_uint64_scale (GST_BUFFER_OFFSET_END (buffer), GST_SECOND,
@@ -234,7 +170,7 @@ GST_START_TEST (test_granulepos_offset)
     gst_buffer_unref (buffer);
   }
 
-  stop_pipeline (bin, pad);
+  gst_buffer_straw_stop_pipeline (bin, pad);
 
   gst_object_unref (pad);
   gst_object_unref (bin);
@@ -249,10 +185,9 @@ GST_START_TEST (test_timestamps)
   gchar *pipe_str;
   GstBuffer *buffer;
   GError *error = NULL;
-  GstClockTime timestamp;
 
   pipe_str = g_strdup_printf ("audiotestsrc"
-      " ! audio/x-raw-int,rate=44100" " ! audioconvert ! vorbisenc ! fakesink");
+      " ! audio/x-raw-int,rate=44100 ! audioconvert ! vorbisenc ! fakesink");
 
   bin = gst_parse_launch (pipe_str, &error);
   fail_unless (bin != NULL, "Error parsing pipeline: %s",
@@ -264,37 +199,38 @@ GST_START_TEST (test_timestamps)
     GstElement *sink = gst_bin_get_by_name (GST_BIN (bin), "fakesink0");
 
     fail_unless (sink != NULL, "Could not get fakesink out of bin");
-    pad = gst_element_get_pad (sink, "sink");
+    pad = gst_element_get_static_pad (sink, "sink");
     fail_unless (pad != NULL, "Could not get pad out of fakesink");
     gst_object_unref (sink);
   }
 
-  start_pipeline (bin, pad);
+  gst_buffer_straw_start_pipeline (bin, pad);
 
   /* check header packets */
-  buffer = get_buffer (bin, pad);
+  buffer = gst_buffer_straw_get_buffer (bin, pad);
   check_buffer_timestamp (buffer, GST_CLOCK_TIME_NONE);
   check_buffer_duration (buffer, GST_CLOCK_TIME_NONE);
   check_buffer_granulepos (buffer, 0);
   gst_buffer_unref (buffer);
 
-  buffer = get_buffer (bin, pad);
+  buffer = gst_buffer_straw_get_buffer (bin, pad);
   check_buffer_timestamp (buffer, GST_CLOCK_TIME_NONE);
   check_buffer_duration (buffer, GST_CLOCK_TIME_NONE);
   check_buffer_granulepos (buffer, 0);
   gst_buffer_unref (buffer);
 
-  buffer = get_buffer (bin, pad);
+  buffer = gst_buffer_straw_get_buffer (bin, pad);
   check_buffer_timestamp (buffer, GST_CLOCK_TIME_NONE);
   check_buffer_duration (buffer, GST_CLOCK_TIME_NONE);
   check_buffer_granulepos (buffer, 0);
   gst_buffer_unref (buffer);
 
   {
-    GstClockTime next_timestamp, last_granulepos;
+    GstClockTime next_timestamp;
+    gint64 last_granulepos;
 
     /* first buffer has timestamp 0 */
-    buffer = get_buffer (bin, pad);
+    buffer = gst_buffer_straw_get_buffer (bin, pad);
     last_granulepos = GST_BUFFER_OFFSET_END (buffer);
     check_buffer_timestamp (buffer, 0);
     /* don't really have a good way of checking duration... */
@@ -305,7 +241,7 @@ GST_START_TEST (test_timestamps)
     gst_buffer_unref (buffer);
 
     /* check continuity with the next buffer */
-    buffer = get_buffer (bin, pad);
+    buffer = gst_buffer_straw_get_buffer (bin, pad);
     check_buffer_timestamp (buffer, next_timestamp);
     check_buffer_duration (buffer,
         gst_util_uint64_scale (GST_BUFFER_OFFSET_END (buffer), GST_SECOND,
@@ -317,7 +253,7 @@ GST_START_TEST (test_timestamps)
     gst_buffer_unref (buffer);
   }
 
-  stop_pipeline (bin, pad);
+  gst_buffer_straw_stop_pipeline (bin, pad);
 
   gst_object_unref (pad);
   gst_object_unref (bin);
@@ -325,15 +261,133 @@ GST_START_TEST (test_timestamps)
 
 GST_END_TEST;
 
-Suite *
+static gboolean
+drop_second_data_buffer (GstPad * droppad, GstBuffer * buffer, gpointer unused)
+{
+  return !(GST_BUFFER_OFFSET (buffer) == 1024);
+}
+
+GST_START_TEST (test_discontinuity)
+{
+  GstElement *bin;
+  GstPad *pad, *droppad;
+  gchar *pipe_str;
+  GstBuffer *buffer;
+  GError *error = NULL;
+  guint drop_id;
+
+  pipe_str = g_strdup_printf ("audiotestsrc samplesperbuffer=1024"
+      " ! audio/x-raw-int,rate=44100" " ! audioconvert ! vorbisenc ! fakesink");
+
+  bin = gst_parse_launch (pipe_str, &error);
+  fail_unless (bin != NULL, "Error parsing pipeline: %s",
+      error ? error->message : "(invalid error)");
+  g_free (pipe_str);
+
+  /* the plan: same as test_timestamps, but dropping a buffer and seeing if
+     vorbisenc correctly notes the discontinuity */
+
+  /* get the pad to use to drop buffers */
+  {
+    GstElement *sink = gst_bin_get_by_name (GST_BIN (bin), "vorbisenc0");
+
+    fail_unless (sink != NULL, "Could not get vorbisenc out of bin");
+    droppad = gst_element_get_static_pad (sink, "sink");
+    fail_unless (droppad != NULL, "Could not get pad out of vorbisenc");
+    gst_object_unref (sink);
+  }
+
+  /* get the pad */
+  {
+    GstElement *sink = gst_bin_get_by_name (GST_BIN (bin), "fakesink0");
+
+    fail_unless (sink != NULL, "Could not get fakesink out of bin");
+    pad = gst_element_get_static_pad (sink, "sink");
+    fail_unless (pad != NULL, "Could not get pad out of fakesink");
+    gst_object_unref (sink);
+  }
+
+  drop_id = gst_pad_add_buffer_probe (droppad,
+      G_CALLBACK (drop_second_data_buffer), NULL);
+  gst_buffer_straw_start_pipeline (bin, pad);
+
+  /* check header packets */
+  buffer = gst_buffer_straw_get_buffer (bin, pad);
+  check_buffer_timestamp (buffer, GST_CLOCK_TIME_NONE);
+  check_buffer_duration (buffer, GST_CLOCK_TIME_NONE);
+  check_buffer_granulepos (buffer, 0);
+  gst_buffer_unref (buffer);
+
+  buffer = gst_buffer_straw_get_buffer (bin, pad);
+  check_buffer_timestamp (buffer, GST_CLOCK_TIME_NONE);
+  check_buffer_duration (buffer, GST_CLOCK_TIME_NONE);
+  check_buffer_granulepos (buffer, 0);
+  gst_buffer_unref (buffer);
+
+  buffer = gst_buffer_straw_get_buffer (bin, pad);
+  check_buffer_timestamp (buffer, GST_CLOCK_TIME_NONE);
+  check_buffer_duration (buffer, GST_CLOCK_TIME_NONE);
+  check_buffer_granulepos (buffer, 0);
+  gst_buffer_unref (buffer);
+
+  /* two phases: continuous granulepos values up to 1024, then a first
+     discontinuous granulepos whose granulepos corresponds to a gap ending at
+     2048. */
+  {
+    GstClockTime next_timestamp = 0;
+    gint64 last_granulepos = 0;
+
+    while (last_granulepos < 1024) {
+      buffer = gst_buffer_straw_get_buffer (bin, pad);
+      last_granulepos = GST_BUFFER_OFFSET_END (buffer);
+      check_buffer_timestamp (buffer, next_timestamp);
+      fail_if (GST_BUFFER_IS_DISCONT (buffer), "expected continuous buffer");
+      next_timestamp += GST_BUFFER_DURATION (buffer);
+      gst_buffer_unref (buffer);
+    }
+
+    fail_unless (last_granulepos == 1024,
+        "unexpected granulepos: %" G_GUINT64_FORMAT, last_granulepos);
+  }
+
+  {
+    buffer = gst_buffer_straw_get_buffer (bin, pad);
+    /* The first buffer after the discontinuity will produce zero output
+     * samples (because of the overlap/add), so it won't increment the 
+     * granulepos, which should be 2048 after the discontinuity.
+     */
+    fail_unless (GST_BUFFER_OFFSET_END (buffer) == 2048,
+        "expected granulepos after gap: %" G_GUINT64_FORMAT,
+        GST_BUFFER_OFFSET_END (buffer));
+    fail_unless (GST_BUFFER_IS_DISCONT (buffer),
+        "expected discontinuous buffer");
+    gst_buffer_unref (buffer);
+  }
+
+  gst_buffer_straw_stop_pipeline (bin, pad);
+  gst_pad_remove_buffer_probe (droppad, drop_id);
+
+  gst_object_unref (droppad);
+  gst_object_unref (pad);
+  gst_object_unref (bin);
+}
+
+GST_END_TEST;
+
+#endif /* #ifndef GST_DISABLE_PARSE */
+
+static Suite *
 vorbisenc_suite (void)
 {
   Suite *s = suite_create ("vorbisenc");
   TCase *tc_chain = tcase_create ("general");
 
   suite_add_tcase (s, tc_chain);
+#ifndef GST_DISABLE_PARSE
   tcase_add_test (tc_chain, test_granulepos_offset);
   tcase_add_test (tc_chain, test_timestamps);
+  tcase_add_test (tc_chain, test_discontinuity);
+#endif
 
   return s;
 }
