@@ -17,7 +17,7 @@ play_file (const gchar * bin, const gint delay, const gchar * uri)
   g_object_set (play, "uri", uri, NULL);
   g_printerr ("Playing %s\n", uri);
   sret = gst_element_set_state (play, GST_STATE_PLAYING);
-  if (sret != GST_STATE_CHANGE_ASYNC && sret != GST_STATE_CHANGE_SUCCESS) {
+  if (sret != GST_STATE_CHANGE_ASYNC) {
     g_printerr ("ERROR: state change failed, sret=%d\n", sret);
     goto next;
   }
@@ -100,10 +100,8 @@ main (int argc, char **argv)
   };
   GTimer *timer;
 
-#if !GLIB_CHECK_VERSION (2, 31, 0)
   if (!g_thread_supported ())
     g_thread_init (NULL);
-#endif
 
   ctx = g_option_context_new ("FILES OR DIRECTORIES WITH AUDIO FILES");
   g_option_context_add_main_entries (ctx, options, NULL);
@@ -115,7 +113,7 @@ main (int argc, char **argv)
   g_option_context_free (ctx);
 
   if (!bin)
-    bin = g_strdup ("playbin");
+    bin = "playbin";
 
   if (strcmp (bin, "playbin") && strcmp (bin, "playbin2")) {
     g_print ("Please provide a valid playbin argument; playbin | playbin2");
@@ -152,7 +150,6 @@ main (int argc, char **argv)
   }
 
   g_strfreev (args);
-  g_free (bin);
   g_timer_destroy (timer);
 
   return 0;
